#!BPY

"""
Name: 'ROBOT PLAN'
Blender: 234
Group: 'Wizards'
Tooltip: 'Create Plan Robot'
"""

######################################################
# ROBOTPLAN.py 
#
# Script for creating a plan robot following an empty
#
#####################################################
# This script creates a robot following a plan robot
# which effector follows an animated empty
# How to use it :
# - Create an empty
# - Make it move according to the desired trajectory
# - Launch the script
# - Set up variables : 
# - Press "Create Robot"
# WARNING : reachable space is to be defined
######################################################
# Author : Jean-Baptiste PERIN (jb_perin@yahoo.fr)
######################################################

import Blender
from Blender import *
from Blender import Mathutils
from Blender.Mathutils import * 
import math
from math import *


#configuration robot
#a1 = 2.0
#a2 = 3.0
#a3 = 3.0


scn = Blender.Scene.getCurrent()

def names(ob): return ob.getName()


#########
# Cette fonction renvoie la liste des empties
# in  : 
# out : emp_list (List of Object) la liste des objets de type "Empty"
#########
def getEmpty(name):
	p = None
	objs = Blender.Object.Get()
	for o in objs:
		if o.getType()=="Empty" and o.getName()==name:
			p = o
	return p


#########
# Cette fonction renvoie un empty 
# in  : objname : le nom de l'empty recherche
# out : myobj : l'empty cree ou retrouve
#########
def GetOrCreateEmpty(objname):
	objects = Blender.Object.Get()
	if objname in map(names,objects):
		myobj = Blender.Object.Get(objname)
		print objname+' exists'
	else:
		myobj = Blender.Object.New('Empty',objname)

		scn.link(myobj)	
		print objname+' was created'
	return myobj

def Main():
	global a1,a2,a3, startframe, endframe, targetname
	print "**** START ****"

	Blender.Set("curframe",startframe)


	ema0=GetOrCreateEmpty('_Hips')
	ema1=GetOrCreateEmpty('_ArmA')
	ema2=GetOrCreateEmpty('_ArmB')
	ema3=GetOrCreateEmpty('_ArmC')
	ema0.clearIpo()
	ema1.clearIpo()
	ema2.clearIpo()
	ema3.clearIpo()
	ema0.makeParent([ema1], 1, 0)
	ema1.makeParent([ema2], 1, 0)
	ema2.makeParent([ema3], 1, 0)
	ema0.setLocation(0.0, 0.0, 0.0)
	ema1.setLocation(0.0, 0.0, a1)
	ema2.setLocation(a2, 0.0, 0.0)
	ema3.setLocation(a3, 0.0, 0.0)
	Blender.Redraw()

	ipoa0 = Ipo.New('Object','ipoa0')
	ipoa1 = Ipo.New('Object','ipoa1')
	ipoa2 = Ipo.New('Object','ipoa2')

	ema0.setIpo(ipoa0)
	ema1.setIpo(ipoa1)
	ema2.setIpo(ipoa2)

	ema0PosY=ipoa0.addCurve('LocY')
	ema1RotY=ipoa1.addCurve('RotY')
	ema2RotY=ipoa2.addCurve('RotY')

	ema0PosY.addBezier((Blender.Get("curframe"), 0))
	ema1RotY.addBezier((Blender.Get("curframe"), 0))
	ema2RotY.addBezier((Blender.Get("curframe"), 0))

	while endframe >= Blender.Get("curframe"):


		# obtention des coordonnees cibles
	
		target = Blender.Object.Get(targetname)
		targetmat = target.getMatrix('worldspace')
		print "target pos = ", targetmat.translationPart()
		targetX = targetmat[3][0]
		targetY = targetmat[3][1]
		targetZ = targetmat[3][2]
	
		# calcul des positions angulaires
		print (targetX**2+(a1-targetZ)**2-a3**2-a2**2)/(2*a3*a2)
		b=acos((targetX**2+(a1-targetZ)**2-a3**2-a2**2)/(2*a3*a2))
		K1=a3*cos(b)+a2
		K2=a3*sin(b)
		unmoinsM2=(-((targetX/K1 - (a1-targetZ)/K2) / (K2/K1+K1/K2)))**2
		a=acos(sqrt(1-unmoinsM2))

		ema0PosY.addBezier((Blender.Get("curframe"), targetY))
		ema1RotY.addBezier((Blender.Get("curframe"), -degrees(a)/10.0))
		ema2RotY.addBezier((Blender.Get("curframe"), degrees(b)/10.0))

		##-----------
		## Passage a la frame suivante
		##-----------
		num_frame = Blender.Get("curframe")
		Blender.Set("curframe", num_frame+1)


	ema0PosY.Recalc()
	ema1RotY.Recalc()
	ema2RotY.Recalc()

	Blender.Redraw()
	Blender.Set("curframe",1)

	print "**** END ****"


########################################################################
#
#            GUI FUNCTIONS AND VARIABLES
#
########################################################################

EFrame = Blender.Draw.Create(0)
SFrame2 = Blender.Draw.Create(0)
TargetName = Blender.Draw.Create(0)
LgA = Blender.Draw.Create(0)
LgB = Blender.Draw.Create(0)
LgC = Blender.Draw.Create(0)
Msg = ' '

def event (evt, val):
	if evt == Blender.Draw.ESCKEY:
		Blender.Draw.Exit()
	return

def button_event(evt):
	global EFrame, SFrame2, LgA, LgB, LgC, Msg, TargetName
	global endframe, startframe, targetname, a1, a2, a3
	if evt==1:
		startframe = eval(SFrame2.val)
		endframe =  eval(EFrame.val)
		A1 = eval(str(LgA.val))
		A2 = eval(str(LgB.val))
		A3 = eval(str(LgC.val))
		targetname = TargetName.val
		if startframe>=endframe:
			Msg = 'Start frame must be lower than End frame'
		else:
			ob = getEmpty(targetname)
			if (ob!=None): 
				Blender.Draw.Exit()
				a1=A1
				a2=A2
				a3=A3
				Main()
			else:
				Msg = 'Empty '+ targetname+ ' not found'
				
		Blender.Draw.Redraw(1)

def GUI():
	global EFrame, SFrame2, TargetName, Msg, LgA, LgB, LgC
	Blender.BGL.glClearColor(0,0,1,1)
	Blender.BGL.glClear(Blender.BGL.GL_COLOR_BUFFER_BIT)
	Blender.BGL.glColor3f(1,1,1)
	Blender.BGL.glRasterPos2i(20,225)
	Blender.Draw.Text ("Robot Plan v0.0 by Jean-Baptiste PERIN", 'normal')
	LgA = Blender.Draw.String("Longueur A: ", -1, 20, 200, 250, 20, '2', 100)
	LgB = Blender.Draw.String("Longueur B: ", -1, 20, 175, 250, 20, '3', 100)
	LgC = Blender.Draw.String("Longueur C: ", -1, 20, 150, 250, 20, '3', 100)
	TargetName = Blender.Draw.String("TargetName: ", -1, 20, 75, 250, 20, 'Target', 100)
	EFrame = Blender.Draw.String("Endframe: ", -1, 20, 100, 250, 20, str(Blender.Get("endframe")), 100)
	SFrame2 = Blender.Draw.String("Startframe: ", -1, 20, 125, 250, 20, str(Blender.Get("staframe")), 100)
	Blender.Draw.Toggle("Create Robot", 1, 20, 10, 100, 20, 0, "Create Robot")
	Blender.BGL.glRasterPos2i(20,40)
	Blender.Draw.Text (Msg, 'normal')

Blender.Draw.Register(GUI, event, button_event)